@@ -59,10 +59,14 @@
 # Check the max number of colours in the palette brewer.pal.info
 # No HCRs is the total number of HCRs - not the number selected
 get_hcr_colours <- function(hcr_names, chosen_hcr_names){
-  #allcols <- colorRampPalette(brewer.pal(11,"PiYG"))(length(hcr_names))
-  #allcols <- colorRampPalette(brewer.pal(12,"Paired"))(length(hcr_names))
-  #allcols <- colorRampPalette(RColorBrewer::brewer.pal(8,"Dark2"))(length(hcr_names))
-  allcols <- colorRampPalette(RColorBrewer::brewer.pal(11,"RdYlBu"))(length(hcr_names))
+  # Looks OK
+  #allcols <- colorRampPalette(RColorBrewer::brewer.pal(11,"RdYlBu"))(length(hcr_names))
+  #  Wes Anderson palette - use Steve Zissou
+  allcols <- wesanderson::wes_palette("Zissou1", length(hcr_names), type = "continuous")
+  # See these notes:
+  #type: Either "continuous" or "discrete". Use continuous if you want
+  #      to automatically interpolate between colours. @importFrom
+  #      graphics rgb rect par image text
   names(allcols) <- hcr_names
   hcrcols <- allcols[chosen_hcr_names]
   return(hcrcols)
@@ -118,6 +122,7 @@ sideways_histogram <- function(dat, range, lhist=20, num.dnorm=5*lhist, dcol="bl
 #' @param app_params A vector of application parameters.
 #' @param timestep The current timestep (optional).
 #' @param show_last Show the previous iters as ghosts (optional).
+#' @param percentile_range A vector of length with minimum and maximum percentile range to plot.
 #' 
 #' @return A plot
 #' @rdname front_page_plots
@@ -564,7 +569,7 @@ plot_hcr_intro_arrow <- function(stock, timestep){
 #' @rdname front_page_plots
 #' @name Front page plots
 #' @export
-plot_metric_with_histo <- function(stock, stock_params, mp_params, metric, app_params=NULL, show_last=TRUE, quantiles=c(0.2,0.8)){
+plot_metric_with_histo <- function(stock, stock_params, mp_params, metric, app_params=NULL, show_last=TRUE, percentile_range = c(20,80)){
   def.par <- par(no.readonly = TRUE) # as seen in layout doc
   on.exit(par(def.par))
   # Plot the metric with an extra sideways histogram
@@ -575,7 +580,7 @@ plot_metric_with_histo <- function(stock, stock_params, mp_params, metric, app_p
   par(mar=c(pext, pext, bspc, bspc), oma=rep(ospc, 4)) # plot parameters
   if (metric == "biomass"){
     # The timeseries of biomass in the big window
-    plot_biomass(stock=stock, stock_params=stock_params, mp_params=mp_params, show_last=show_last, quantiles=quantiles)
+    plot_biomass(stock=stock, stock_params=stock_params, mp_params=mp_params, show_last=show_last, quantiles=percentile_range / 100)
     # The histogram should be the true B/K
     final_yr <- dim(stock$biomass)[2]
     dat <- stock$biomass[,final_yr] / stock_params$k
@@ -583,7 +588,7 @@ plot_metric_with_histo <- function(stock, stock_params, mp_params, metric, app_p
   }
   else if (metric == "catch"){
     # The timeseries of biomass in the big window
-    plot_catch(stock=stock, stock_params=stock_params, mp_params=mp_params, show_last=show_last, quantiles=quantiles)
+    plot_catch(stock=stock, stock_params=stock_params, mp_params=mp_params, show_last=show_last, quantiles=percentile_range / 100)
     final_yr <- dim(stock$catch)[2]
     dat <- stock$catch[,final_yr]
     range <- c(0, get_catch_ymax(stock$catch, mp_params))
@@ -595,7 +600,7 @@ plot_metric_with_histo <- function(stock, stock_params, mp_params, metric, app_p
     max_rel_cpue <- 2
     rel_cpue[rel_cpue > max_rel_cpue] <- max_rel_cpue
     ymax <- max(c(rel_cpue * 1.1, 1.0), na.rm=TRUE)
-    plot_relcpue(stock=stock, stock_params=stock_params, mp_params=mp_params, app_params=app_params, show_last=show_last, quantiles=quantiles, ymax=ymax)
+    plot_relcpue(stock=stock, stock_params=stock_params, mp_params=mp_params, app_params=app_params, show_last=show_last, quantiles=percentile_range / 100, ymax=ymax)
     # data for the histogram
     final_yr <- dim(rel_cpue)[2]
     dat <- rel_cpue[,final_yr]
@@ -842,6 +847,7 @@ plot_yieldcurve_projections <- function(stock, stock_params, app_params){
 #' @param dat The data.frame with the data to be plotted.
 #' @param hcr_choices The names of the HCRs to plot.
 #' @param stock_params A vector of life history and stochasticy parameters.
+#' @param percentile_range A vector of length with minimum and maximum percentile range to plot.
 #' 
 #' @return A ggplot2 plot object.
 #' @rdname comparison_plots
@@ -906,24 +912,24 @@ plot_yieldcurve_projections <- function(stock, stock_params, app_params){
 #' # Table of PIs. Only pass in 1 time period
 #' pitable(dat=subset(pisums$periodqs, period=="Long"))
 #' @export
-plot_majuro <- function(dat, hcr_choices, stock_params){
-
+plot_majuro <- function(dat, percentile_range = c(20,80), hcr_choices, stock_params){
   hcrcols <- get_hcr_colours(hcr_names=unique(dat$hcrref), chosen_hcr_names=hcr_choices)
-
-  # Need a dataset with X20 X80 Y20 Y80
-  majdat <- subset(dat, pi %in% c("ffmsy", "sbsbf0"))
+  # Need a dataset with percentiles
+  majdat <- subset(dat, piname %in% c("F/FMSY", "SB/SBF=0"))
   # Need to shunt the years by 1 as B in year Y is the result of F in year Y-1 
   # So add 1 to the F years
   majdat[majdat$pi=="ffmsy","year"] <- majdat[majdat$pi=="ffmsy","year"] + 1
-  majdat <- dplyr::select(majdat, pi, year, X20., X80., X50., hcrref)
+  majdat <- dplyr::select(majdat, c("pi", "year", paste("X",percentile_range[1],".",sep=""), paste("X",percentile_range[2],".",sep=""), X50., hcrref))
+  # Rename for simplicity
+  majdat <- dplyr::rename(majdat, "min" = paste("X",percentile_range[1],".",sep=""), "max" = paste("X",percentile_range[2],".",sep=""), "med" = "X50.")
   majdat <- tidyr::gather(majdat, key="XX", value="data", -pi, -year, -hcrref)
   majdat$pix <- paste(majdat$pi,majdat$XX,sep="")
   majdat <- tidyr::spread(dplyr::select(majdat, year, hcrref, data, pix), key="pix", value="data")
   # Remove NA years
-  majdat <- subset(majdat, !(is.na(ffmsyX50.) | is.na(sbsbf0X50.)))
+  majdat <- subset(majdat, !(is.na(ffmsymed) | is.na(biomassmed)))
 
   lrp <- stock_params[["lrp"]]
-  ymax <- max(max(majdat$ffmsyX80., na.rm=TRUE) * 0.1, 2.0)
+  ymax <- max(max(majdat$ffmsymax, na.rm=TRUE) * 0.1, 2.0)
   
   p <- ggplot(majdat)
   # Big red
@@ -936,18 +942,18 @@ plot_majuro <- function(dat, hcr_choices, stock_params){
   # Put the lines on - Do twice, once in fat black , then thin with colour
   # Lines and crosses
   # Fat black
-  p <- p + geom_line(aes(x=sbsbf0X50., y=ffmsyX50.), colour="black", size=2)
-  p <- p + geom_errorbar(aes(x=sbsbf0X50., ymin=ffmsyX20., ymax=ffmsyX80., group=year), colour="black", size=2)
-  p <- p + geom_errorbarh(aes(y=ffmsyX50., xmin=sbsbf0X20., xmax=sbsbf0X80., group=year), colour="black", size=2)
+  p <- p + geom_line(aes(x=biomassmed, y=ffmsymed), colour="black", size=2)
+  p <- p + geom_errorbar(aes(x=biomassmed, ymin=ffmsymin, ymax=ffmsymax, group=year), colour="black", size=2)
+  p <- p + geom_errorbarh(aes(y=ffmsymed, xmin=biomassmin, xmax=biomassmax, group=year), colour="black", size=2)
   # Thin colour
-  p <- p + geom_line(aes(x=sbsbf0X50., y=ffmsyX50., colour=hcrref), size=1.3)
-  p <- p + geom_errorbar(aes(x=sbsbf0X50., ymin=ffmsyX20., ymax=ffmsyX80., group=year, colour=hcrref), size=1.3)
-  p <- p + geom_errorbarh(aes(y=ffmsyX50., xmin=sbsbf0X20., xmax=sbsbf0X80., group=year, colour=hcrref), size=1.3)
+  p <- p + geom_line(aes(x=biomassmed, y=ffmsymed, colour=hcrref), size=1.3)
+  p <- p + geom_errorbar(aes(x=biomassmed, ymin=ffmsymin, ymax=ffmsymax, group=year, colour=hcrref), size=1.3)
+  p <- p + geom_errorbarh(aes(y=ffmsymed, xmin=biomassmin, xmax=biomassmax, group=year, colour=hcrref), size=1.3)
   # Black point
-  p <- p + geom_point(aes(x=sbsbf0X50., y=ffmsyX50.))
+  p <- p + geom_point(aes(x=biomassmed, y=ffmsymed))
   # Final point in white
   maxyear <- max(majdat$year)
-  p <- p + geom_point(data=subset(majdat, year==maxyear), aes(x=sbsbf0X50., y=ffmsyX50.), colour="white")
+  p <- p + geom_point(data=subset(majdat, year==maxyear), aes(x=biomassmed, y=ffmsymed), colour="white")
 
   p <- p + scale_colour_manual(values=hcrcols)
   p <- p + xlab("SB/SBF=0") + ylab("F/FMSY")
@@ -987,29 +993,34 @@ plot_majuro <- function(dat, hcr_choices, stock_params){
 #' @export
 quantile_plot <- function(dat, hcr_choices, wormdat=NULL,
                           alpha20_80 = 0.6, linetype_worm=1,
+                          percentile_range = c(20,80),
                           colour_worm="black",
-                          size_worm=0.2, add_start_line=TRUE, time_period_lines=TRUE, short_term = 2016:2024, medium_term = 2025:2033, long_term = 2034:2042, last_plot_year=2042, show_spaghetti=FALSE){
-
+                          size_worm=0.3, add_start_line=TRUE, time_period_lines=TRUE, short_term = 2016:2024, medium_term = 2025:2033, long_term = 2034:2042, last_plot_year=2042, show_spaghetti=FALSE){
   # Sort out colours based on chosen HCRs
   hcrcols <- get_hcr_colours(hcr_names=unique(dat$hcrref), chosen_hcr_names=hcr_choices)
   # Select the chosen HCRs only - could do this in the call to plot in app?
   dat <- subset(dat, hcrref %in% hcr_choices)
+  # Rename for percentiles reference
+  dat <- dplyr::rename(dat, "min" = paste("X",percentile_range[1],".",sep=""), "max" = paste("X",percentile_range[2],".",sep="")) 
   # Chop out NA rows
   dat <- dat[!is.na(dat$X50.),]
   # Start the plot
   p <- ggplot(dat, aes(x=year))
   # Ribbons and lines
-  p <- p + geom_ribbon(aes(ymin=X20., ymax=X80., fill=hcrref), alpha=alpha20_80)
-  p <- p + geom_line(aes(y=X80., group=hcrref), colour="black")
-  p <- p + geom_line(aes(y=X20., group=hcrref), colour="black")
+  p <- p + geom_ribbon(aes(ymin=min, ymax=max, fill=hcrref), alpha=alpha20_80)
+  p <- p + geom_line(aes(y=max, group=hcrref), colour="black", size=ggplot2::rel(0.5))
+  p <- p + geom_line(aes(y=min, group=hcrref), colour="black", size=ggplot2::rel(0.5))
   # Add median line
-  p <- p + geom_line(aes(y=X50., group=hcrref), colour="black", linetype=2, size=1)
+  #p <- p + geom_line(aes(y=X50., group=hcrref), colour="black", linetype=2, size=1)
+  p <- p + geom_line(aes(y=X50., group=hcrref), colour="black", linetype=2, size=ggplot2::rel(0.5))
+
   # Plotting options
   p <- p + xlab("Year")
   # Add worms
   if (!is.null(wormdat) & show_spaghetti==TRUE){
     wormdat <- subset(wormdat, hcrref %in% hcr_choices)
     wormdat <- wormdat[!is.na(wormdat$data),]
+    p <- p + geom_line(data=wormdat, aes(x=year, y=data, group=wormid), colour="black", linetype=linetype_worm, size=size_worm*1.1)
     p <- p + geom_line(data=wormdat, aes(x=year, y=data, group=wormid, colour=hcrref), linetype=linetype_worm, size=size_worm)
   }
   ## Colours
@@ -1052,8 +1063,6 @@ myboxplot <- function(dat, hcr_choices, plot_type="median_bar"){
   hcrcols <- get_hcr_colours(hcr_names=unique(dat$hcrref), chosen_hcr_names=hcr_choices)
   # Fucking subset and fucking non standard evaluation - why is this so shit??????
   dat <- subset(dat, hcrref %in% hcr_choices)
-  #dat <- dat[dat$hcrref %in% hcr_choices,]
-
   if (plot_type=="median_bar"){
     p <- ggplot(dat, aes(x=period, y=X50., fill=hcrref))
     p <- p + geom_bar(stat="identity", position="dodge", colour="black", width=0.7)
@@ -1104,10 +1113,13 @@ myradar <- function(dat, hcr_choices, scaling="scale", polysize=2){
       dat <- dplyr::group_by(dat, period, pi)
       dat <- dplyr::mutate(dat, value = order(X50.) / length(hcr_choices))
     }
-    # Reorder PI
-    dat <- dat[order(dat$piname),]
 
-    p <- ggplot(data=dat, aes(x=piname, y=value,group=hcrref))
+    # Need to wrap text of piname
+    max_len <- 10 # max length of label in characters
+    dat$piname_wrap <- sapply(dat$piname, function(y) paste(strwrap(y, max_len), collapse = "\n"), USE.NAMES = FALSE)
+    dat <- dat[order(dat$piname_wrap),]
+
+    p <- ggplot(data=dat, aes(x=piname_wrap, y=value,group=hcrref))
     p <- p + geom_polygon(aes(fill=hcrref), colour="black", alpha=0.6, size=polysize)
     p <- p + xlab("") + ylab("") + theme(legend.position="bottom", legend.title=element_blank())
     p <- p + facet_wrap(~period)
@@ -1117,40 +1129,15 @@ myradar <- function(dat, hcr_choices, scaling="scale", polysize=2){
     p <- p + theme(axis.text.y=element_blank(), axis.ticks.y=element_blank()) # Remove y axis 
     p <- p + ylim(0,1)
     #p + coord_polar() # For a weird fat look!
-
-    p <- p + theme(axis.text=element_text(size=16), axis.title=element_text(size=16), strip.text=element_text(size=16), legend.text=element_text(size=16))
+    p <- p + theme(#axis.text=element_text(size=16),
+                axis.text.x  = element_text(size = ggplot2::rel(1.5)), # Make x axes text smaller
+                   axis.title=element_text(size=16),
+                   strip.text=element_text(size=16),
+                   legend.text=element_text(size=16))
 
     return(p)
 }
 
-# Not a plot - but included here anyway
-# The big PI tables
-
-#' pitable
-#'
-#' pitable() is not a plot but a table comparing PIs across HCRs and periods. Only pass in 1 time period at a time.
-#'
-#' @return A data.frame to be shown as a table.
-#' @rdname comparison_plots
-#' @name Comparison plots
-#' @export
-pitable <- function(dat){
-    # Rows are the PIs, columns are the HCRs
-    signif <- 2
-    dat$value <- paste(round(dat$X50.,signif), " (", round(dat$X20., signif), ",", round(dat$X80., signif), ")", sep="")
-    # Fix pi1
-    dat[dat$pi=="pi1", "value"] <- round(dat[dat$pi=="pi1", "X50."],signif)
-    tabdat <- dat[,c("hcrref", "piname", "value")]
-    tabdat[tabdat$name=="Biomass","piname"] <- "SB/SBF=0"
-    tabdat <- as.data.frame(tidyr::spread(tabdat, key="hcrref", value="value"))
-    # Have rownames?
-    #rnames <- tabdat[,1]
-    #tabdat <- tabdat[,-1]
-    #rownames(tabdat) <- rnames
-    colnames(tabdat)[1] <- "Indicator"
-    return(tabdat)
-}
-
 
 
 #-------------------------------------------------------------------------------------
